@@ -23,8 +23,7 @@
 import os
 from os.path import join, exists, isdir, isfile, dirname, abspath, expanduser
 from shutil import copy as copyfile
-import gtk, gnome.ui 	 
-from gtk import glade
+import gtk	 
 import gettext
 import locale
 	 
@@ -77,6 +76,7 @@ if not exists(HAMSTER_DB):
 
 dispatcher = Dispatcher()
 storage = None
+trace_sql = False
     
 # Path to images, icons
 ART_DATA_DIR = join(SHARED_DATA_DIR, "art")
