@@ -1,6 +1,6 @@
 # - coding: utf-8 -
 
-# Copyright (C) 2007, 2008 Toms Bauģis <toms.baugis at gmail.com>
+# Copyright (C) 2007-2009 Toms Bauģis <toms.baugis at gmail.com>
 # Copyright (C) 2007 Patryk Zawadzki <patrys at pld-linux.org>
 
 # This file is part of Project Hamster.
@@ -25,6 +25,7 @@ try:
     import sqlite3 as sqlite
 except ImportError:
     try:
+        print "Using sqlite2"
         from pysqlite2 import dbapi2 as sqlite
     except ImportError:
         print "Error: Neither sqlite3 nor pysqlite2 found"
@@ -33,8 +34,9 @@ import os, time
 import datetime
 import hamster
 import hamster.storage
+from hamster import stuff
 import datetime as dt
-
+import copy
         
 class Storage(hamster.storage.Storage):
     # we are saving data under $HOME/.gnome2/hamster-applet/hamster.db
@@ -48,6 +50,9 @@ class Storage(hamster.storage.Storage):
         activity = self.fetchone("select name from activities where id = ?", (id, ))
         existing_id = self.__get_activity_by_name(activity['name'], category_id)
         
+        if id == existing_id: # we are already there, go home
+            return False
+        
         if existing_id: #ooh, we have something here!
             # first move all facts that belong to movable activity to the new one
             update = """
@@ -72,6 +77,8 @@ class Storage(hamster.storage.Storage):
             """
             
             self.execute(statement, (category_id, max_order, id))
+        
+        return True
     
     def __add_category(self, name):
         new_rec = self.fetchone("select max(id) +1, max(category_order) + 1  from categories")
@@ -133,8 +140,8 @@ class Storage(hamster.storage.Storage):
                      ORDER BY deleted, id desc
                         LIMIT 1
             """
-            
-            res = self.fetchone(query, (name,))
+
+            res = self.fetchone(query, (name, ))
         
         if res:
             # if the activity was marked as deleted, ressurect on first call
@@ -173,12 +180,14 @@ class Storage(hamster.storage.Storage):
                           a.start_time AS start_time,
                           a.end_time AS end_time,
                           a.description as description,
-                          b.name AS name, b.id as activity_id
+                          b.name AS name, b.id as activity_id,
+                          coalesce(c.name, ?) as category, coalesce(c.id, -1) as category_id
                      FROM facts a
                 LEFT JOIN activities b ON a.activity_id = b.id
+                LEFT JOIN categories c on b.category_id = c.id
                     WHERE a.id = ? 
         """
-        return self.fetchone(query, (id,))
+        return self.fetchone(query, (_("Unsorted"), id))
 
     def __get_last_activity(self):
         query = """
@@ -186,15 +195,25 @@ class Storage(hamster.storage.Storage):
                           a.start_time AS start_time,
                           a.end_time AS end_time,
                           a.description as description,
-                          b.name AS name, b.id as activity_id
+                          b.name AS name, b.id as activity_id,
+                          coalesce(c.name, ?) as category, coalesce(c.id, -1) as category_id
                      FROM facts a
                 LEFT JOIN activities b ON a.activity_id = b.id
+                LEFT JOIN categories c on b.category_id = c.id
                     WHERE date(a.start_time) = ?
                  ORDER BY a.start_time desc
                     LIMIT 1
         """
-        return self.fetchone(query, (dt.date.today(), ))
+        last = self.fetchone(query, (_("Unsorted"), dt.date.today()))
+        if not last:
+            #try yesterday if there is nothing today
+            last = self.fetchone(query, (_("Unsorted"),
+                                         dt.date.today() - dt.timedelta(days=1)))
 
+        if last and last["end_time"]: #will consider as last only if it is going on
+           last = None
+        
+        return last
     def __touch_fact(self, fact, end_time):
         # tasks under one minute do not count
         if end_time - fact['start_time'] < datetime.timedelta(minutes = 1):
@@ -207,104 +226,111 @@ class Storage(hamster.storage.Storage):
             """
             self.execute(query, (end_time, fact['id']))
 
+    def __solve_overlaps(self, start_time, end_time):
+        """finds facts that happen in given interval and shifts them to
+        make room for new fact"""
+        
+        # in case of missing end_time, treat is as ongoing task, but only
+        # if that's not too far in past
+        if not end_time and (dt.datetime.now() - start_time <= dt.timedelta(days=1)):
+            end_time = dt.datetime.now()
+            
+        
+        # activities that we are overlapping.
+        # second OR clause is for elimination - |new fact--|---old-fact--|--new fact|
+        query = """
+                   SELECT a.*, b.name
+                     FROM facts a
+                LEFT JOIN activities b on b.id = a.activity_id
+                    WHERE ((start_time < ? and end_time > ?)
+                           OR (start_time < ? and end_time > ?))
+                           
+                       OR ((start_time < ? and start_time > ?)
+                           OR (end_time < ? and end_time > ?))
+                 ORDER BY start_time
+                """
+        conflicts = self.fetchall(query, (start_time, start_time, end_time, end_time,
+                                          end_time, start_time, end_time, start_time))
+        
+        for fact in conflicts:
+            # split - truncate until beginning of new entry and create new activity for end
+            if fact["start_time"] < start_time < fact["end_time"] and \
+               fact["start_time"] < end_time < fact["end_time"]:
+                
+                print "splitting %s" % fact["name"]
+                self.execute("""UPDATE facts
+                                   SET end_time = ?
+                                 WHERE id = ?""", (start_time, fact["id"]))
+                self.__add_fact(fact["name"], end_time, fact["end_time"])
+
+            #eliminate
+            elif fact["end_time"] and \
+                 start_time < fact["start_time"] < end_time and \
+                 start_time < fact["end_time"] < end_time:
+                print "eliminating %s" % fact["name"]
+                self.__remove_fact(fact["id"])
+            
+            # overlap start
+            elif start_time < fact["start_time"] < end_time:
+                print "Overlapping start of %s" % fact["name"]
+                self.execute("UPDATE facts SET start_time=? WHERE id=?",
+                             (end_time, fact["id"]))
+            
+            # overlap end
+            elif start_time < fact["end_time"] < end_time:
+                print "Overlapping end of %s" % fact["name"]
+                self.execute("UPDATE facts SET end_time=? WHERE id=?",
+                             (start_time, fact["id"]))
+
+
     def __add_fact(self, activity_name, start_time = None, end_time = None):
-        start_time = start_time or datetime.datetime.now()
-        
-        #see if we have description of activity somewhere here (delimited by comma)
-        description = None
-        if activity_name.find(",") > 0:
-            activity_name, description = activity_name.split(",", 1)
-            description = description.strip()
-        
+        activity = stuff.parse_activity_input(activity_name)
+        start_time = activity.start_time or start_time or datetime.datetime.now()
+        end_time = activity.end_time or end_time
+
+            
         # now check if maybe there is also a category
         category_id = None
-        if activity_name.find("@") > 0:
-            #at symbol marks category
-            activity_name, category_name = activity_name.split("@", 1)
-            
-            if category_name:
-                category_name = category_name.strip()
-                category_id = self.__get_category_by_name(category_name)
-                if not category_id:
-                    category_id = self.__add_category(category_name)
+        if activity.category_name:
+            category_id = self.__get_category_by_name(activity.category_name)
+            if not category_id:
+                category_id = self.__add_category(activity.category_name)
         
         # try to find activity
-        activity_id = self.__get_activity_by_name(activity_name, category_id)
-
+        activity_id = self.__get_activity_by_name(activity.activity_name,
+                                                  category_id)
         if not activity_id:
-            activity_id = self.__add_activity(activity_name, category_id)
+            activity_id = self.__add_activity(activity.activity_name,
+                                              category_id)
 
 
-        # now fetch facts for the specified day and check if we have to
-        # split or change span 
-        day_facts = self.__get_facts(start_time.date())
-        for fact in day_facts:
-            # first check if maybe we are overlapping end
-            if fact['end_time'] and fact['start_time'] < start_time < fact['end_time']:
-                #set fact's end time to our start one
-                update = """
-                           UPDATE facts
-                              SET end_time = ?
-                            WHERE id = ?
-                """
-                self.execute(update, (start_time, fact["id"]))
+        # if we are working on +/- current day - check the last_activity
+        if (dt.datetime.now() - start_time <= dt.timedelta(days=1)):
+            last_activity = self.__get_last_activity()
 
-                # now check - maybe we are inside the fact. in that case
-                # we should create another task after our one
-                if end_time and end_time < fact['end_time']:
-                    self.__add_fact(fact['name'], end_time, fact['end_time'])
-
-           
-            else: #end's fine? what about start then?
-                if fact['end_time'] and end_time and fact['start_time'] < end_time < fact['end_time'] \
-                or not fact['end_time'] and end_time and fact['start_time'] < end_time: # case for the entry before the last one
-                    #set fact's start time to our end one
-                    update = """
-                               UPDATE facts
-                                  SET start_time = ?
-                                WHERE id = ?
-                    """
-                    self.execute(update, (end_time, fact["id"]))
-
-        # now check if maybe we are at the last task, and if that's true
-        # look if we maybe have to finish it or delete if it was too short
-        if day_facts:
-            last_fact = day_facts[-1]
-            
-            if last_fact['end_time'] == None and last_fact['start_time'] < start_time:
+            if last_activity and last_activity['start_time'] < start_time:
                 #if this is the same, ongoing activity, then there is no need to create another one
-                if last_fact['activity_id'] == activity_id:
-                    return last_fact
+                if not activity.description \
+                   and last_activity['activity_id'] == activity_id:
+                    return last_activity
                 
-                
-                delta = (start_time - last_fact['start_time'])
-            
-                if 60 >= delta.seconds >= 0:
-                    self.__remove_fact(last_fact['id'])
-                    start_time = last_fact['start_time']
-
-                    # go further now and check, maybe task before the last one
-                    # is the same we have now. If that happened before less
-                    # than a minute - remove end time!
-                    if len(day_facts) > 2 and day_facts[-2]['name'] == activity_name:
-                        print day_facts[-2]['name'], activity_name, day_facts[-2]['end_time'], start_time
-                        delta = (start_time - day_facts[-2]['end_time'])
-                        if 60 >= delta.seconds >= 0:
-                            update = """
-                                        UPDATE facts
-                                           SET end_time = null
-                                         WHERE id = ?
-                                    """
-                            self.execute(update, (day_facts[-2]["id"],))
-                            return day_facts[-2]
+                if not activity.description \
+                   and not last_activity["description"] \
+                   and 60 >= (start_time - last_activity['start_time']).seconds >= 0:
+                    self.__remove_fact(last_activity['id'])
+                    start_time = last_activity['start_time']
                 else:
-                    #set previous fact end time
+                    #stop 
                     update = """
                                UPDATE facts
                                   SET end_time = ?
                                 WHERE id = ?
                     """
-                    self.execute(update, (start_time, fact["id"]))
+                    self.execute(update, (start_time, last_activity["id"]))
+
+
+        #done with the current activity, now we can solve overlaps
+        self.__solve_overlaps(start_time, end_time)
 
 
         # finally add the new entry
@@ -312,7 +338,7 @@ class Storage(hamster.storage.Storage):
                     INSERT INTO facts (activity_id, start_time, end_time, description)
                                VALUES (?, ?, ?, ?)
         """
-        self.execute(insert, (activity_id, start_time, end_time, description))
+        self.execute(insert, (activity_id, start_time, end_time, activity.description))
 
         fact_id = self.fetchone("select max(id) as max_id from facts")['max_id']
         
@@ -326,17 +352,137 @@ class Storage(hamster.storage.Storage):
                           a.end_time AS end_time,
                           a.description as description,
                           b.name AS name, b.id as activity_id,
-                          coalesce(c.name, ?) as category, c.id as category_id
+                          coalesce(c.name, ?) as category, coalesce(c.id, -1) as category_id
                      FROM facts a
                 LEFT JOIN activities b ON a.activity_id = b.id
                 LEFT JOIN categories c on b.category_id = c.id
-                    WHERE date(a.start_time) >= ? and date(a.start_time) <= ?
+                    WHERE (date(a.start_time) >= ? and date(a.start_time) <= ?)
+                       OR (date(a.end_time) >= ? and date(a.end_time) <= ?)
                  ORDER BY a.start_time
         """
+        end_date = end_date or date
+        
+        facts = self.fetchall(query, (_("Unsorted"), date, end_date, date, end_date))
+        res = []
+
+        today = dt.date.today()
+        yesterday = dt.date.today() - dt.timedelta(days=1)
+        now = dt.datetime.now()
+        
+
+        # fetch last activity here - we will be looking it up for comparisons
+        last_activity = None
+        if date >= today >= end_date or abs(date-today).days < 2 \
+                                   or abs(end_date-today).days < 2:
+            last_activity = self.__get_last_activity()
+
+        # deal with late-night workers!
+        if not facts and date == today:
+            # no facts today?! let's get last fact of yesterday
+            if last_activity:
+                # last fact has not finished, we think that it is still ongoing
+                f = dict(
+                    id = last_activity["id"],
+                    start_time = dt.datetime.combine(date, dt.time(0,0)),
+                    end_time = now,
+                    description = last_activity["description"],
+                    name = last_activity["name"],
+                    activity_id = last_activity["activity_id"],
+                    category = last_activity["category"],
+                    category_id = last_activity["category_id"],
+                    delta = now - dt.datetime.combine(date, dt.time(0,0))
+                )
+            
+                return [f]
+
+
+        for fact in facts:
+            fact_start_date = fact["start_time"].date()
+            if fact["end_time"]:
+                fact_end_date = fact["end_time"].date()
+            else:
+                fact_end_date = None
+                                   
+            f = dict(
+                id = fact["id"],
+                start_time = fact["start_time"],
+                end_time = fact["end_time"],
+                description = fact["description"],
+                name = fact["name"],
+                activity_id = fact["activity_id"],
+                category = fact["category"],
+                category_id = fact["category_id"]
+            )
+            
+            if not fact_end_date:
+                if fact_start_date == today and fact["start_time"] < now:
+                    # today, present
+                    f["delta"] = now -  fact["start_time"]
+                elif last_activity and fact_start_date == yesterday and f["id"] == last_activity["id"]:
+                    # last fact and it is in yesterday - split it!
+                    #first yesterday until midnight
+                    f["end_time"] = dt.datetime.combine(today, dt.time(0,0))
+                    f["delta"] = f["end_time"] - f["start_time"]
+                    res.append(f)
+                    
+                    #and now today until now
+                    f = copy.copy(f)
+                    f["start_time"] = dt.datetime.combine(today, dt.time(0, 0))
+                    f["end_time"] = now
+                    f["delta"] = f["end_time"] - f["start_time"]
+                else:
+                    f["delta"] = None
+
+                res.append(f)
+            elif fact_start_date != fact_end_date:
+                # check if maybe we have to split activity in two
+                if date <= fact["start_time"].date()  <= end_date:
+                    start_fact = copy.copy(f)
+                    start_fact["end_time"] = dt.datetime.combine(f["end_time"],
+                                                                 dt.time(0, 0))
+                    start_fact["delta"] = start_fact["end_time"] - start_fact["start_time"]
+                    res.append(start_fact)
+
+                if date <= fact["end_time"].date()  <= end_date:
+                    end_fact = copy.copy(f)
+                    end_fact["start_time"] = dt.datetime.combine(f["end_time"],
+                                                                 dt.time(0, 0))
+                    end_fact["delta"] = end_fact["end_time"] - end_fact["start_time"]
+                    res.append(end_fact)
+            else:
+                #else is we have end date and it is the same date
+                f["delta"] = fact["end_time"] - fact["start_time"]
+                res.append(f)
+
+        return res
+
+    def __get_popular_categories(self):
+        """returns categories used in the specified interval"""
+        query = """
+                   SELECT coalesce(c.name, ?) as category, count(a.id) as popularity
+                     FROM facts a
+                LEFT JOIN activities b on a.activity_id = b.id
+                LEFT JOIN categories c on c.id = b.category_id
+                 GROUP BY b.category_id
+                 ORDER BY popularity desc
+        """
+        return self.fetchall(query, (_("Unsorted"), ))
+
+    def __get_interval_activity_ids(self, date, end_date = None):
+        """returns activities used in the specified interval"""
+        query = """
+                   SELECT a.name, coalesce(b.name, ?) as category_name
+                     FROM activities a
+                LEFT JOIN categories b on b.id = a.category_id
+                    WHERE a.id in (SELECT activity_id from facts
+                                    WHERE (date(start_time) >= ? and date(start_time) <= ?)
+                                       OR (date(end_time) >= ? and date(end_time) <= ?))
+        """
         end_date = end_date or date        
 
-        return self.fetchall(query, (_("Unsorted"), date, end_date))
-
+        return self.fetchall(query, (_("Unsorted"), date, end_date, date, end_date))
+        
+    
     def __remove_fact(self, fact_id):
         query = """
                    DELETE FROM facts
@@ -425,13 +571,12 @@ class Storage(hamster.storage.Storage):
         self.execute("delete from categories where id = ?", (id, ))
         
     
-    def __swap_activities(self, id1, id2):
+    def __swap_activities(self, id1, priority1, id2, priority2):
         """ swaps nearby activities """
         # TODO - 2 selects and 2 updates is wrong we could live without selects
-        priority1 = self.fetchone("select activity_order from activities where id = ?", (id1,))[0]
-        priority2 = self.fetchone("select activity_order from activities where id = ?", (id2,))[0]
-        self.execute("update activities set activity_order = ? where id = ?", (priority1, id2) )
-        self.execute("update activities set activity_order = ? where id = ?", (priority2, id1) )
+        self.execute(["update activities set activity_order = ? where id = ?",
+                      "update activities set activity_order = ? where id = ?"],
+                      [(priority1, id2), (priority2, id1)])
 
     def __add_activity(self, name, category_id = None):
         # first check that we don't have anything like that yet
@@ -462,7 +607,7 @@ class Storage(hamster.storage.Storage):
 
     """ Here be dragons (lame connection/cursor wrappers) """
     def get_connection(self):
-        if self.con == None:
+        if self.con is None:
             self.con = sqlite.connect(hamster.HAMSTER_DB, detect_types=sqlite.PARSE_DECLTYPES|sqlite.PARSE_COLNAMES)
             self.con.row_factory = sqlite.Row
 
@@ -474,7 +619,8 @@ class Storage(hamster.storage.Storage):
         con = self.connection
         cur = con.cursor()
 
-        print query, params
+        if hamster.trace_sql:
+            print query, params
 
         if params:
             cur.execute(query, params)
@@ -494,15 +640,25 @@ class Storage(hamster.storage.Storage):
             return None
 
     def execute(self, statement, params = ()):
+        """execute sql statement. optionally you can give multiple statements
+        to save on cursor creation and closure"""
         con = self.connection
         cur = con.cursor()
-
-        print statement, params
-        res = cur.execute(statement, params)
+        
+        if isinstance(statement, list) == False: #we kind of think that we will get list of instructions
+            statement = [statement]
+            params = [params]
+            
+        if isinstance(statement, list):
+            for i in range(len(statement)):
+                if hamster.trace_sql:
+                    print statement[i], params[i]
+         
+                res = cur.execute(statement[i], params[i])
 
         con.commit()
         cur.close()
-
+        
     def run_fixtures(self):
         # defaults
         work_category = {"name": _("Work"),
@@ -697,13 +853,13 @@ class Storage(hamster.storage.Storage):
         category_count = self.fetchone("select count(*) from categories")[0]
         
         if category_count == 0:
-            work_cat_id = self.__add_category(_(work_category["name"]))
+            work_cat_id = self.__add_category(work_category["name"])
             for entry in work_category["entries"]:
-                self.__add_activity(_(entry), work_cat_id)
+                self.__add_activity(entry, work_cat_id)
         
-            nonwork_cat_id = self.__add_category(_(nonwork_category["name"]))
+            nonwork_cat_id = self.__add_category(nonwork_category["name"])
             for entry in nonwork_category["entries"]:
-                self.__add_activity(_(entry), nonwork_cat_id)
+                self.__add_activity(entry, nonwork_cat_id)
         
         
         
